@@ -21,8 +21,11 @@
 
 ## phi: the IV estimator of phi(y)
 ## alpha:  the Tikhonov regularization parameter
+## phi.mat: the matrix with colums phi_1, phi_2 etc. over all iterations
 ## num.iterations:  the number of Landweber-Fridman iterations
 ## norm.stop: the vector of values of the objective function used for stopping
+## norm.value: the norm not multiplied by the number of iterations
+## convergence: a character string indicating whether/why iteration terminated
 
 crsiv <- function(y,
                   z,
@@ -35,10 +38,14 @@ crsiv <- function(y,
                   alpha.min=1.0e-10,
                   alpha.max=1.0e-01,
                   alpha.tol=.Machine$double.eps^0.25,
+                  deriv=0,
                   iterate.max=1000,
-                  iterate.tol=1.0e-04,
                   iterate.diff.tol=1.0e-08,
                   constant=0.5,
+                  penalize.iteration=TRUE,
+                  smooth.residuals=TRUE,
+                  start.from=c("Eyz","EEywz"),
+                  starting.values=NULL,
                   stop.on.increase=TRUE,
                   method=c("Landweber-Fridman","Tikhonov"),
                   opts=list("MAX_BB_EVAL"=10000,
@@ -47,7 +54,6 @@ crsiv <- function(y,
                             "MIN_MESH_SIZE"=paste("r",sqrt(.Machine$double.eps),sep=""),
                             "MIN_POLL_SIZE"=paste("r",sqrt(.Machine$double.eps),sep=""),
                             "DISPLAY_DEGREE"=0),
-                  deriv=0,
                   ...) {
 
   crs.messages <- getOption("crs.messages")
@@ -55,89 +61,89 @@ crsiv <- function(y,
   ## This function was constructed initially by Samuele Centorrino
   ## <samuele.centorrino@univ-tlse1.fr>
   ## the following papers:
-  
+
   ## A) Econometrica (forthcoming, article date February 25 2011)
-  
+
   ## "Nonparametric Instrumental Regression"
   ## S. Darolles, Y. Fan, J.P. Florens, E. Renault
-  
+
   ## B) Econometrics Journal (2010), volume 13, pp. S1–S27. doi:
   ## 10.1111/j.1368-423X.2010.00314.x
-  
+
   ## "The practice of non-parametric estimation by solving inverse
   ## problems: the example of transformation models"
-  
+
   ## FREDERIQUE FEVE AND JEAN-PIERRE FLORENS
   ## IDEI and Toulouse School of Economics, Universite de Toulouse
   ## Capitole 21 alle de de Brienne, 31000 Toulouse, France. E-mails:
   ## feve@cict.fr, florens@cict.fr
-  
+
   ## It was modified by Jeffrey S. Racine <racinej@mcmaster.ca> and all
   ## errors remain my responsibility. I am indebted to Samuele and the
   ## Toulouse School of Economics for their generous hospitality.
-  
+
   ## First we require two functions, the first that conducts Regularized
   ## Tikhonov Regression' (aka Ridge Regression)
-  
+
   ## This function conducts regularized Tikhonov regression which
   ## corresponds to (3.9) in Feve & Florens (2010).
-  
+
   ## This function accepts as arguments
-  
+
   ## alpha: penalty
   ## CZ:    row-normalized kernel weights for the `independent' variable
   ## CY:    row-normalized kernel weights for the `dependent' variable
   ## Cr:    row-normalized kernel weights for the `instrument/endogenous' variable (see NOTE below)
   ## r:     vector of conditional expectations (z can be E(Z|z) - see NOTE below)
-  
+
   ## NOTE: for Cr, in the transformation model case treated in Feve &
   ## Florens (2010) this maps Z onto the Y space. In the IV case
   ## (Darrolles, Fan, Florens & Renault (2011, forthcoming Econometrica)
   ## it maps W (the instrument) onto the space of the endogenous
   ## regressor Z.
-  
+
   ## NOTE: for r, in the transformation model it will be equivalent to
   ## the vector of exogenous covariates, and in the endogenous case r is
   ## the conditional mean of y given the instrument W.
-  
+
   ## This function returns TBA (need better error checking!)
-  
+
   ## phi:   the vector of estimated values for the unknown function at the evaluation points
-  
+
   tikh <- function(alpha,CZ,CY,Cr.r){
     return(solve(alpha*diag(length(Cr.r)) + CY%*%CZ) %*% Cr.r) ## This must be computable via ridge... step 1, step 2, same alpha...
   }
 
   ## This function applies the iterated Tikhonov approach which
   ## corresponds to (3.10) in Feve & Florens (2010).
-  
+
   ## This function accepts as arguments
-  
+
   ## alpha: penalty
   ## CZ:    row-normalized kernel weights for the `independent' variable
   ## CY:    row-normalized kernel weights for the `dependent' variable
   ## Cr:    row-normalized kernel weights for the `instrument/endogenous' variable (see NOTE below)
   ## r:     vector of conditional expectations (z can be E(Z|z) - see NOTE below)
-  
+
   ## NOTE: for Cr, in the transformation model case treated in Feve &
   ## Florens (2010) this maps Z onto the Y space. In the IV case
   ## (Darrolles, Fan, Florens & Renault (2011, forthcoming Econometrica)
   ## it maps W (the instrument) onto the space of the endogenous
   ## regressor Z.
-  
+
   ## NOTE: for r, in the transformation model it will be equivalent to
   ## the vector of exogenous covariates, and in the endogenous case r is
   ## the conditional mean of y given the instrument W.
-  
+
   ## This function returns TBA (need better error checking!)
-  
+
   ## phi:   the vector of estimated values for the unknown function at the evaluation points
-  
+
   ## SSalpha: (scalar) value of the sum of square residuals criterion
   ## which is a function of alpha (see (3.10) of Feve & Florens (2010)
 
   ## Cr.r is always E.E.y.w.z, r is always E.y.w
-  
+
   ittik <- function(alpha,CZ,CY,Cr.r,r) {
     invmat <- solve(alpha*diag(length(Cr.r)) + CY%*%CZ)
     tikh.val <- invmat %*% Cr.r
@@ -149,6 +155,7 @@ crsiv <- function(y,
 
   ## Basic error checking
 
+  start.from <- match.arg(start.from)
   if(!is.logical(stop.on.increase)) stop("stop.on.increase must be logical (TRUE/FALSE)")
 
   if(missing(y)) stop("You must provide y")
@@ -159,7 +166,6 @@ crsiv <- function(y,
   if(!is.null(x) && NROW(y) != NROW(x)) stop("y and x have differing numbers of rows")
   if(iterate.max < 2) stop("iterate.max must be at least 2")
   if(constant <= 0 || constant >=1) stop("constant must lie in (0,1)")
-  if(iterate.tol <= 0) stop("iterate.tol must be positive")
   if(iterate.diff.tol < 0) stop("iterate.diff.tol must be non-negative")
 
   ## Cast as data frames
@@ -182,8 +188,8 @@ crsiv <- function(y,
 
   wnames <- names(w)
   znames <- names(z)
-  names(weval) <- wnames  
-  names(zeval) <- znames  
+  names(weval) <- wnames
+  names(zeval) <- znames
 
   ## If there exist exogenous regressors X, append these to the
   ## formulas involving Z (can be manually added to W by the user if
@@ -191,7 +197,7 @@ crsiv <- function(y,
 
   if(!is.null(x)) {
     xnames <- names(x)
-    names(xeval) <- xnames    
+    names(xeval) <- xnames
   }
 
   ## Now create evaluation data
@@ -200,31 +206,30 @@ crsiv <- function(y,
     traindata <- data.frame(y,z,w)
     evaldata <- data.frame(zeval,weval)
   } else {
-    traindata <- data.frame(y,z,w,x)    
+    traindata <- data.frame(y,z,w,x)
     evaldata <- data.frame(zeval,weval,xeval)
   }
 
   formula.yw <- as.formula(paste("y ~ ", paste(wnames, collapse= "+")))
-  formula.phiw <- as.formula(paste("phi ~ ", paste(wnames, collapse= "+")))  
-  formula.residw <- as.formula(paste("(y-phi.j.m.1) ~ ", paste(wnames, collapse= "+")))
-  formula.residphi0w <- as.formula(paste("residuals(phi.0) ~ ", paste(wnames, collapse= "+")))    
+  formula.phiw <- as.formula(paste("phi ~ ", paste(wnames, collapse= "+")))
+  formula.residw <- as.formula(paste("(y-phi) ~ ", paste(wnames, collapse= "+")))
 
   if(is.null(x)) {
     formula.yz <- as.formula(paste("y ~ ", paste(znames, collapse= "+")))
     formula.Eywz <- as.formula(paste("E.y.w ~ ", paste(znames, collapse= "+")))
-    formula.Ephiwz <- as.formula(paste("E.phi.w ~ ", paste(znames, collapse= "+")))  
-    formula.predictmodelresidphi0z <- as.formula(paste("predict(model.residphi0,newdata=evaldata) ~ ", paste(znames, collapse= "+")))
-    formula.predictmodelresidwz <- as.formula(paste("predict(model.residw,newdata=evaldata) ~ ", paste(znames, collapse= "+")))
+    formula.Ephiwz <- as.formula(paste("E.phi.w ~ ", paste(znames, collapse= "+")))
+    formula.residwz <- as.formula(paste("residw ~ ", paste(znames, collapse= "+")))
   } else {
     formula.yz <- as.formula(paste("y ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))
     formula.Eywz <- as.formula(paste("E.y.w ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))
-    formula.Ephiwz <- as.formula(paste("E.phi.w ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))  
-    formula.predictmodelresidphi0z <- as.formula(paste("predict(model.residphi0,newdata=evaldata) ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))
-    formula.predictmodelresidwz <- as.formula(paste("predict(model.residw,newdata=evaldata) ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))
+    formula.Ephiwz <- as.formula(paste("E.phi.w ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))
+    formula.residwz <- as.formula(paste("residw ~ ", paste(znames, collapse= "+"), " + ", paste(xnames, collapse= "+")))
   }
 
+  if(!is.null(starting.values) && (NROW(starting.values) != NROW(evaldata))) stop(paste("starting.values must be of length",NROW(evaldata)))
+
   if(method=="Tikhonov") {
-  
+
     ## Now y=phi(z) + u, hence E(y|w)=E(phi(z)|w) so we need two
     ## bandwidths, one for y on w and one for phi(z) on w (in the
     ## first step we use E(y|w) as a proxy for phi(z) and use
@@ -234,21 +239,21 @@ crsiv <- function(y,
     ## but value is required
 
     convergence <- NULL
-    
+
     ## First we conduct the regression spline estimator of y on w
-    
+
     console <- printClear(console)
     console <- printPop(console)
     console <- printPush("Computing weights and optimal smoothing for E(y|w)...", console)
     if(crs.messages) options(crs.messages=FALSE)
     model<-crs(formula.yw,opts=opts,data=traindata,...)
-    if(crs.messages) options(crs.messages=TRUE)    
-    E.y.w <- predict(model,newdata=evaldata)
+    if(crs.messages) options(crs.messages=TRUE)
+    E.y.w <- predict(model,newdata=evaldata,...)
     B <- model.matrix(model$model.lm)
     KYW <- B%*%solve(t(B)%*%B)%*%t(B)
-   
+
     ## Next, we conduct the regression spline of E(y|w) on z
-    
+
     console <- printClear(console)
     console <- printPop(console)
     if(is.null(x)) {
@@ -258,27 +263,27 @@ crsiv <- function(y,
     }
     if(crs.messages) options(crs.messages=FALSE)
     model <- crs(formula.Eywz,opts=opts,data=traindata,...)
-    if(crs.messages) options(crs.messages=TRUE)    
-    E.E.y.w.z <- predict(model,newdata=evaldata)
+    if(crs.messages) options(crs.messages=TRUE)
+    E.E.y.w.z <- predict(model,newdata=evaldata,...)
     B <- model.matrix(model$model.lm)
     KYWZ <- B%*%solve(t(B)%*%B)%*%t(B)
-    
+
     ## Next, we minimize the function ittik to obtain the optimal value
     ## of alpha (here we use the iterated Tikhonov function) to
     ## determine the optimal alpha for the non-iterated scheme. Note
     ## that the function `optimize' accepts bounds on the search (in
     ## this case alpha.min to alpha.max))
-    
+
     ## E(r|z)=E(E(phi(z)|w)|z)
     ## \phi^\alpha = (\alpha I+CzCw)^{-1}Cr x r
-    
+
     if(is.null(alpha)) {
       console <- printClear(console)
       console <- printPop(console)
       console <- printPush("Numerically solving for alpha...", console)
       alpha <- optimize(ittik, c(alpha.min,alpha.max), tol = alpha.tol, CZ = KYW, CY = KYWZ, Cr.r = E.E.y.w.z, r = E.y.w)$minimum
     }
-    
+
     ## Finally, we conduct regularized Tikhonov regression using this
     ## optimal alpha to get a first stage estimate of phi
 
@@ -292,11 +297,11 @@ crsiv <- function(y,
     phi <- as.vector(tikh(alpha, CZ = KYW, CY = KYWZ, Cr.r = E.E.y.w.z))
 
     ## KYWZ and KZWS no longer used, save memory
-    
+
     rm(KYW,KYWZ)
 
-    ## Conduct kernel regression of phi(z) on w  
-    
+    ## Conduct kernel regression of phi(z) on w
+
     console <- printClear(console)
     console <- printPop(console)
     if(is.null(x)) {
@@ -306,13 +311,13 @@ crsiv <- function(y,
     }
     if(crs.messages) options(crs.messages=FALSE)
     model <- crs(formula.phiw,opts=opts,data=traindata,...)
-    if(crs.messages) options(crs.messages=TRUE)    
-    E.phi.w <- predict(model,newdata=evaldata)
+    if(crs.messages) options(crs.messages=TRUE)
+    E.phi.w <- predict(model,newdata=evaldata,...)
     B <- model.matrix(model$model.lm)
     KPHIW <- B%*%solve(t(B)%*%B)%*%t(B)
-    
+
     ## Conduct kernel regression of E(phi(z)|w) on z
-    
+
     console <- printClear(console)
     console <- printPop(console)
     if(is.null(x)) {
@@ -322,21 +327,21 @@ crsiv <- function(y,
     }
     if(crs.messages) options(crs.messages=FALSE)
     model <- crs(formula.Ephiwz,opts=opts,data=traindata,...)
-    if(crs.messages) options(crs.messages=TRUE)    
+    if(crs.messages) options(crs.messages=TRUE)
     B <- model.matrix(model$model.lm)
     KPHIWZ <- B%*%solve(t(B)%*%B)%*%t(B)
-    
+
     ## Next, we minimize the function ittik to obtain the optimal value of
     ## alpha (here we use the iterated Tikhonov approach) to determine the
     ## optimal alpha for the non-iterated scheme.
-    
+
     if(is.null(alpha)) {
       console <- printClear(console)
       console <- printPop(console)
       console <- printPush("Iterating and computing the numerical solution for alpha...", console)
       alpha <- optimize(ittik,c(alpha.min,alpha.max), tol = alpha.tol, CZ = KPHIW, CY = KPHIWZ, Cr.r = E.E.y.w.z, r = E.y.w)$minimum
     }
-    
+
     ## Finally, we conduct regularized Tikhonov regression using this
     ## optimal alpha.
 
@@ -348,13 +353,13 @@ crsiv <- function(y,
       console <- printPush("Computing final phi(z,x) estimate...", console)
     }
     phi <- as.vector(tikh(alpha, CZ = KPHIW, CY = KPHIWZ, Cr.r = E.E.y.w.z))
-    
+
     console <- printClear(console)
     console <- printPop(console)
-    
+
     if((alpha-alpha.min)/alpha.min < 0.01) warning(paste(" Tikhonov parameter alpha (",formatC(alpha,digits=4,format="f"),") is close to the search minimum (",alpha.min,")",sep=""))
     if((alpha.max-alpha)/alpha.max < 0.01) warning(paste(" Tikhonov parameter alpha (",formatC(alpha,digits=4,format="f"),") is close to the search maximum (",alpha.max,")",sep=""))
-    
+
     ## phi.0 is the conditional mean model. We compute lambda =
     ## fitted(phi.0)-phi then transform y via
     ## y.lambda=y-lambda. Here we overwrite y so that we can reuse the
@@ -379,27 +384,39 @@ crsiv <- function(y,
                  segments=phi.0$segments,
                  lambda=phi.0$lambda,
                  include=phi.0$include,
-                 kernel=phi.0$kernel, 
+                 kernel=phi.0$kernel,
                  basis=phi.0$basis,
                  knots=phi.0$knots,
                  tau=phi.0$tau,
                  deriv=deriv,
                  data=traindata)
-    if(crs.messages) options(crs.messages=TRUE)    
+    if(crs.messages) options(crs.messages=TRUE)
 
     model$residuals <- residuals.phi
     model$phi <- phi
     model$alpha <- alpha
 
     return(model)
-    
+
   } else {
-    
+
     ## Landweber-Fridman
 
-    ## We begin the iteration computing phi.0 and phi.1 directly, then
-    ## iterate.
-    
+    ## Create storage vector/matrix
+
+    norm.stop <- numeric()
+
+    ## Compute E(Y|w) for the stopping rule
+
+    console <- printClear(console)
+    console <- printPop(console)
+    console <- printPush(paste("Computing optimal smoothing and E(Y|w) for the stopping rule...",sep=""),console)
+
+    if(crs.messages) options(crs.messages=FALSE)
+    model.E.y.w <- crs(formula.yw,opts=opts,data=traindata,...)
+    E.y.w <- predict(model.E.y.w,newdata=evaldata,...)
+    if(crs.messages) options(crs.messages=TRUE)
+
     console <- printClear(console)
     console <- printPop(console)
     if(is.null(x)) {
@@ -407,29 +424,63 @@ crsiv <- function(y,
     } else {
       console <- printPush(paste("Computing optimal smoothing and phi(z,x) for iteration 1...",sep=""),console)
     }
-    if(crs.messages) options(crs.messages=FALSE)
-    phi.0 <- crs(formula.yz,opts=opts,data=traindata,...)
-    model.residphi0 <- crs(formula.residphi0w,opts=opts,data=traindata,...)
-    model.Eresidphi0.z <- crs(formula.predictmodelresidphi0z,opts=opts,data=traindata,...)
-    if(crs.messages) options(crs.messages=TRUE)    
-    phi.j.m.1 <- predict(phi.0,newdata=evaldata) + constant*predict(model.Eresidphi0.z,newdata=evaldata)
 
-    ## For the stopping rule
+    ## Initial value taken from E(E(Y|w)|z) or E(Y|z) or overridden
+    ## and passed in, formulae all operate on phi. phi.0.NULL flag set
+
+    if(crs.messages) options(crs.messages=FALSE)
+    if(is.null(starting.values)) {
+      phi.0.NULL <- TRUE
+      phi.0 <- crs(formula.yz,opts=opts,data=traindata,...)
+      ## First compute phi.0 (not passed in) then phi
+      if(start.from == "Eyz") {
+        ## Start from E(Y|z)
+        phi <- predict(phi.0,newdata=evaldata,...)
+      } else {
+        ## Start from E(E(Y|w)|z)
+        E.y.w <- fitted(crs(formula.yw,opts=opts,data=traindata,...))
+        model.E.E.y.w.z <- crs(formula.Eywz,opts=opts,data=traindata,...)
+        phi <- predict(model.E.E.y.w.z,newdata=evaldata,...)
+      }
+    } else {
+      phi.0.NULL <- FALSE
+      phi.0.input <- starting.values
+      ## First compute phi (passed in) then phi.0
+      phi <- starting.values
+      phi.0 <- crs(formula.yz,opts=opts,data=traindata,...)
+    }
+
+    starting.values.phi <- phi
+    
+    if(crs.messages) options(crs.messages=TRUE)
 
     console <- printClear(console)
     console <- printPop(console)
-    console <- printPush(paste("Computing optimal smoothing for the stopping rule...",sep=""),console)
-
-    norm.stop <- numeric()
+    if(is.null(x)) {
+      console <- printPush(paste("Computing optimal smoothing for E(Y-phi(z)|w) for iteration 1...",sep=""),console)
+    } else {
+      console <- printPush(paste("Computing optimal smoothing  for E(Y-phi(z,x)|w) for iteration 1...",sep=""),console)
+    }
     if(crs.messages) options(crs.messages=FALSE)
-    model.E.y.w <- crs(formula.yw,opts=opts,data=traindata,...)
-    E.y.w <- predict(model.E.y.w,newdata=evaldata)
-    phi <- phi.j.m.1
-    model.E.phi.w <- crs(formula.phiw,opts=opts,data=traindata,...)
-    E.phi.w <- predict(model.E.phi.w,newdata=evaldata)
-    if(crs.messages) options(crs.messages=TRUE)    
+    if(smooth.residuals) {
+      model.residw <- crs(formula.residw,opts=opts,data=traindata,...)
+      residw <- predict(model.residw,newdata=evaldata,...)
+      model.predict.residw.z <- crs(formula.residwz,opts=opts,data=traindata,...)
+    } else {
+      model.E.phi.w <- crs(formula.phiw,opts=opts,data=traindata,...)
+      residw <- predict(model.E.y.w,newdata=evaldata,...)-predict(model.E.phi.w,newdata=evaldata,...)
+      model.predict.residw.z <- crs(formula.residwz,opts=opts,data=traindata,...)
+    }
+    if(crs.messages) options(crs.messages=TRUE)
 
-    norm.stop[1] <- mean(((E.y.w-E.phi.w)/E.y.w)^2)
+    if(phi.0.NULL) {
+      phi <- predict(phi.0,newdata=evaldata,...) + constant*predict(model.predict.residw.z,newdata=evaldata,...)
+    } else {
+      phi <- phi.0.input + constant*predict(model.predict.residw.z,newdata=evaldata,...)
+    }
+
+    phi.mat <- phi
+    norm.stop[1] <- sum(residw^2)/sum(E.y.w^2)
 
     for(j in 2:iterate.max) {
 
@@ -442,59 +493,81 @@ crsiv <- function(y,
       }
 
       if(crs.messages) options(crs.messages=FALSE)
-      model.residw <- crs(formula.residw,opts=opts,data=traindata,...)
-      model.predict.residw.z <- crs(formula.predictmodelresidwz,opts=opts,data=traindata,...)
-      if(crs.messages) options(crs.messages=TRUE)    
-
-      phi <- phi.j.m.1 + constant*predict(model.predict.residw.z,newdata=evaldata)
-
-      console <- printClear(console)
-      console <- printPop(console)
-      console <- printPush(paste("Computing stopping rule for iteration ", j,"...",sep=""),console)
-
-      ## For the stopping rule (use same smoothing as original,
-      ## cv="none" so no ... args allowed hence pass all relevant args)
-      if(crs.messages) options(crs.messages=FALSE)
-      model.stop <- crs(formula.phiw,
-                        cv="none",
-                        degree=model.E.phi.w$degree,
-                        segments=model.E.phi.w$segments,
-                        lambda=model.E.phi.w$lambda,
-                        include=model.E.phi.w$include,
-                        kernel=model.E.phi.w$kernel, 
-                        basis=model.E.phi.w$basis,
-                        knots=model.E.phi.w$knots,
-                        tau=model.E.phi.w$tau,                        
-                        data=traindata)
-      if(crs.messages) options(crs.messages=TRUE)    
-      
-      E.phi.w <- predict(model.stop,newdata=evaldata)
-      norm.stop[j] <- mean(((E.y.w-E.phi.w)/E.y.w)^2)
-
-      ## If stopping rule criterion increases or we are below stopping
-      ## tolerance then break
-
-      if(norm.stop[j] < iterate.tol) {
-        convergence <- "ITERATE_TOL"
-        break()
+      if(smooth.residuals) {
+        model.residw <- crs(formula.residw,opts=opts,data=traindata,...)
+        residw <- predict(model.residw,newdata=evaldata,...)
+        model.predict.residw.z <- crs(formula.residwz,opts=opts,data=traindata,...)
+      } else {
+        model.E.phi.w <- crs(formula.phiw,opts=opts,data=traindata,...)
+        residw <- predict(model.E.y.w,newdata=evaldata,...)-predict(model.E.phi.w,newdata=evaldata,...)
+        model.predict.residw.z <- crs(formula.residwz,opts=opts,data=traindata,...)
       }
-      if(stop.on.increase && norm.stop[j] > norm.stop[j-1]) {
-        convergence <- "STOP_ON_INCREASE"
-        phi <- phi.j.m.1
-        break()
-      }
-      if(abs(norm.stop[j-1]-norm.stop[j]) < iterate.diff.tol) {
-        convergence <- "ITERATE_DIFF_TOL"
-        break()
+      if(crs.messages) options(crs.messages=TRUE)
+
+      phi <- phi + constant*predict(model.predict.residw.z,newdata=evaldata,...)
+      phi.mat <- cbind(phi.mat,phi)
+
+      norm.stop[j] <- ifelse(penalize.iteration,j*sum(residw^2)/sum(E.y.w^2),sum(residw^2)/sum(E.y.w^2))
+
+      ## The number of iterations in LF is asymptotically equivalent
+      ## to 1/alpha (where alpha is the regularization parameter in
+      ## Tikhonov).  Plus the criterion function we use is increasing
+      ## for very small number of iterations. So we need a threshold
+      ## after which we can pretty much confidently say that the
+      ## stopping criterion is decreasing.  In Darolles et al. (2011)
+      ## \alpha ~ O(N^(-1/(min(beta,2)+2)), where beta is the so
+      ## called qualification of your regularization method. Take the
+      ## worst case in which beta = 0 and then the number of
+      ## iterations is ~ N^0.5.
+
+      if(j > round(sqrt(nrow(traindata))) && !is.monotone.increasing(norm.stop)) {
+
+        ## If stopping rule criterion increases or we are below stopping
+        ## tolerance then break
+
+        if(stop.on.increase && norm.stop[j] > norm.stop[j-1]) {
+          convergence <- "STOP_ON_INCREASE"
+          break()
+        }
+        if(abs(norm.stop[j-1]-norm.stop[j]) < iterate.diff.tol) {
+          convergence <- "ITERATE_DIFF_TOL"
+          break()
+        }
+
       }
 
-      ## For next iteration phi.j
-
-      phi.j.m.1 <- phi
       convergence <- "ITERATE_MAX"
 
     }
 
+    norm.value <- norm.stop/(1:length(norm.stop))
+
+    ## Extract minimum, and check for monotone increasing function and
+    ## issue warning in that case. Otherwise allow for an increasing
+    ## then decreasing (and potentially increasing thereafter) portion
+    ## of the stopping function, ignore the initial increasing portion,
+    ## and take the min from where the initial inflection point occurs
+    ## to the length of norm.stop
+
+    if(which.min(norm.stop) == 1 && is.monotone.increasing(norm.stop)) {
+      warning("Stopping rule increases monotonically (consult model$norm.stop):\nThis could be the result of an inspired initial value (unlikely)\nNote: we suggest manually choosing phi.0 and restarting (e.g. instead set `starting.values' to E[E(Y|w)|z])")
+      convergence <- "FAILURE_MONOTONE_INCREASING"
+      ## Ignore the initial increasing portion, take the min to the
+      ## right of where the initial inflection point occurs
+      j <- 1
+      while(norm.value[j+1] > norm.value[j]) j <- j + 1
+      j <- j-1 + which.min(norm.value[j:length(norm.value)])
+      phi <- phi.mat[,j]
+#      phi <- starting.values.phi
+    } else {
+      ## Ignore the initial increasing portion, take the min to the
+      ## right of where the initial inflection point occurs
+      j <- 1
+      while(norm.stop[j+1] > norm.stop[j]) j <- j + 1
+      j <- j-1 + which.min(norm.stop[j:length(norm.stop)])
+      phi <- phi.mat[,j]
+    }
+
     ## phi.0 is the conditional mean model. We compute lambda =
     ## fitted(phi.0)-phi then transform y via
     ## y.lambda=y-lambda. Here we overwrite y so that we can reuse the
@@ -513,20 +586,22 @@ crsiv <- function(y,
                  segments=phi.0$segments,
                  lambda=phi.0$lambda,
                  include=phi.0$include,
-                 kernel=phi.0$kernel, 
+                 kernel=phi.0$kernel,
                  basis=phi.0$basis,
                  knots=phi.0$knots,
-                 tau=phi.0$tau,                 
+                 tau=phi.0$tau,
                  deriv=deriv,
                  data=traindata)
-    if(crs.messages) options(crs.messages=TRUE)    
+    if(crs.messages) options(crs.messages=TRUE)
 
     model$residuals <- residuals.phi
     model$phi <- phi
+    model$phi.mat <- phi.mat
     model$num.iterations <- j
     model$norm.stop <- norm.stop
+    model$norm.value <- norm.value
     model$convergence <- convergence
-
+    model$starting.values.phi <- starting.values.phi
     console <- printClear(console)
     console <- printPop(console)
 
@@ -535,5 +610,5 @@ crsiv <- function(y,
     return(model)
 
   }
-  
+
 }
