@@ -1,11 +1,12 @@
 /*-------------------------------------------------------------------------------------*/
-/*  NOMAD - Nonsmooth Optimization by Mesh Adaptive Direct search - version 3.5        */
+/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.5.1        */
 /*                                                                                     */
-/*  Copyright (C) 2001-2010  Mark Abramson        - the Boeing Company, Seattle        */
+/*  Copyright (C) 2001-2012  Mark Abramson        - the Boeing Company, Seattle        */
 /*                           Charles Audet        - Ecole Polytechnique, Montreal      */
 /*                           Gilles Couture       - Ecole Polytechnique, Montreal      */
 /*                           John Dennis          - Rice University, Houston           */
 /*                           Sebastien Le Digabel - Ecole Polytechnique, Montreal      */
+/*                           Christophe Tribes    - Ecole Polytechnique, Montreal      */
 /*                                                                                     */
 /*  funded in part by AFOSR and Exxon Mobil                                            */
 /*                                                                                     */
@@ -423,13 +424,17 @@ namespace NOMAD {
     /// Access to the directions.
     /**
        - The computed directions already include Delta^k_m.
-       \param dirs       List of directions                  -- \b OUT.
-       \param poll       Type of poll (primary or secondary) -- \b IN.
-       \param mesh_index Mesh index ell                      -- \b IN.
+       \param dirs          List of directions                     -- \b OUT.
+       \param poll          Type of poll (primary or secondary)    -- \b IN.
+       \param poll_center   Poll center                            -- \b IN.
+       \param first_success First success of the run (can be NULL) -- \b IN.
+       \param mesh_index    Mesh index ell                         -- \b IN.
     */
-    void get_directions ( std::list<NOMAD::Direction> & dirs       ,
-			  NOMAD::poll_type              poll       ,
-			  int                           mesh_index   ) const;
+    void get_directions ( std::list<NOMAD::Direction> & dirs          ,
+			  NOMAD::poll_type              poll          ,
+			  const NOMAD::Point          & poll_center   ,
+			  const NOMAD::Point          * first_success ,
+			  int                           mesh_index      ) const;
     
     /// Access to one direction for a given mesh.
     /**
