@@ -1,11 +1,12 @@
 /*-------------------------------------------------------------------------------------*/
-/*  NOMAD - Nonsmooth Optimization by Mesh Adaptive Direct search - version 3.5        */
+/*  NOMAD - Nonlinear Optimization by Mesh Adaptive Direct search - version 3.5.1        */
 /*                                                                                     */
-/*  Copyright (C) 2001-2010  Mark Abramson        - the Boeing Company, Seattle        */
+/*  Copyright (C) 2001-2012  Mark Abramson        - the Boeing Company, Seattle        */
 /*                           Charles Audet        - Ecole Polytechnique, Montreal      */
 /*                           Gilles Couture       - Ecole Polytechnique, Montreal      */
 /*                           John Dennis          - Rice University, Houston           */
 /*                           Sebastien Le Digabel - Ecole Polytechnique, Montreal      */
+/*                           Christophe Tribes    - Ecole Polytechnique, Montreal      */
 /*                                                                                     */
 /*  funded in part by AFOSR and Exxon Mobil                                            */
 /*                                                                                     */
@@ -40,7 +41,7 @@
   \see    Stats.hpp
 */
 #include "Stats.hpp"
-using namespace std;
+using namespace std;  //zhenghua
 /*---------------------------------------------------------*/
 /*                     affectation operator                */
 /*---------------------------------------------------------*/
@@ -282,7 +283,7 @@ void NOMAD::Stats::display ( const NOMAD::Display & out ) const
 	<< "number of ext. poll successes   : " << _ext_poll_succ       << std::endl
 	<< "number of ext. poll points      : " << _ext_poll_pts        << std::endl
 	<< "number of ext. poll bb eval     : " << _ext_poll_bb_eval    << std::endl;  
-  out << "number of speculatives searches : "   << _nb_spec_searches    << std::endl;
+  out << "number of speculative searches  : "   << _nb_spec_searches    << std::endl;
   if ( _nb_spec_searches > 0 )
     out << "speculative search successes    : " << _spec_success        << std::endl
 	<< "speculative search points       : " << _spec_pts            << std::endl;
